{.$DEFINE DEBUG}

unit CanApp;

interface

uses SysUtils, Classes, Variants, HTTPApp, CGIHTTP, ISAPIApp;//, CGIApp;

type
  TCGIScriptRequest = class(TCGIRequest)
  public
    function MapPath(const LogicalPath: string): string; // Yanlis calisiyor.
    function TranslateURI(const URI: string): string; override;
  end;

  TCGIScriptResponse = class(TCGIResponse)

  end;

  TContentStream = class(TMemoryStream)
  public
    function WriteString(const AString: string): Boolean;
  end;

  TCanCGIApplication = class(TObject)
  private
    FContentStream: TContentStream;
    FRequest: TCGIScriptRequest;
    FResponse: TCGIScriptResponse;
  public
    constructor Create;
    destructor Destroy; override;
    procedure HandleRequest;
    procedure Run;
    procedure Write(Data: Variant);
    function WriteString(const AString: string): Boolean;
    function MapPath(const LogicalPath: string): string;
    procedure Redirect(const URI: string);
    procedure ContentType(const ContentType: string);
    procedure WWWAuthenticate(const Authenticate: string);
    procedure Realm(const ARealm: string);
    procedure StatusCode(const Value: Integer);
    procedure SendStream(AStream: TStream);
    property ContentStream: TContentStream read FContentStream;
    property Request: TCGIScriptRequest read FRequest;
    property Response: TCGIScriptResponse read FResponse;
  end;

var
  Application: TCanCGIApplication;

implementation

uses BrkrConst, Interprt, Execution, SysUnit, UrlMon;//, Shlwapi;

{$IFDEF DEBUG}
const
  CodeText: array[ecNop..ecContinue] of string = (
   'NOP', 'HTML', 'ASSIGN', 'CALL', 'WHILE', 'ENDWHILE', 'REPEAT',
   'UNTIL', 'IF', 'ELSE', 'ENDIF', 'JUMP', 'BREAK', 'CONTINUE' );

function CodeToText(Code: Integer): string;
begin
  if (Code >= ecNop) or (Code <= ecContinue) then
    Result := CodeText[Code]
  else Result := '';
end;

procedure DebugPrint;
var
 F: Text;
 ItemText, Other: string;
 I: Integer;
 Item: TExecutionItem;
begin
  AssignFile( F, 'x_' + MakeSessionID( 8 ) + '.txt' );
  Rewrite( F );
  for I := 0 to ExecutionList.Count -1 do
  begin
    Item := ExecutionList[I];
    ItemText := Format( '%0.4d', [I] );
    ItemText := ItemText + ' ' + CodeToText(Item.Code);
    Other := '';
    case Item.Code of
      ecHtml:;
      ecJump:
        Other := IntToStr( Item.JumpCode );
      ecIf:
        Other := Item.Text + ', ' + IntToStr( Item.JumpCode );
      ecCall:
        Other := Item.Text;
      ecAssign:
        Other := Item.Ident + ', ' + Item.Text;
      ecWhile:
        Other := Item.Text + ', ' + IntToStr( Item.JumpCode );
      ecElse:
        Other := Item.Text;
    end;
    WriteLn( F, ItemText, ' ':8-Length(Other), Other );
  end;
  CloseFile( F );
end;
{$ENDIF}

procedure HandleServerException(E: TObject);
var
  ResultText, ResultHeaders, EMsg: string;
begin
  if E is Exception then
    EMsg := Exception(E).Message
  else
    EMsg := '';
  ResultText := Format(sInternalServerError, [E.ClassName, EMsg]);
  ResultHeaders := Format(
    'Status: 500 %s'#13#10+               { do not localize }
    'Content-Type: text/html'#13#10 +     { do not localize }
    'Content-Length: %d'#13#10#13#10,      { do not localize }
    [EMsg, Length(ResultText)]); { do not localize }
  if IsConsole then
  begin
    FileWrite(TTextRec(Output).Handle, Pointer(ResultHeaders)^, Length(ResultHeaders));
    FileWrite(TTextRec(Output).Handle, Pointer(ResultText)^, Length(ResultText));
  end;
end;

{ TCanCGIApplication }

procedure TCanCGIApplication.ContentType(const ContentType: string);
begin
  Response.ContentType := ContentType;
end;

constructor TCanCGIApplication.Create;
begin
  inherited Create;
  FContentStream := TContentStream.Create;
end;

destructor TCanCGIApplication.Destroy;
begin
  //FContentStream.Free;
  inherited Destroy;
end;

procedure TCanCGIApplication.HandleRequest;

  function GetScriptName: string;
  begin
    if ParamCount = 0 then Result := Request.GetStringVariable(5)
    else Result := ParamStr(1);
  end;

var
  Interpreter: TInterpreter;

begin
  Interpreter := TInterpreter.Create;
  try
    Interpreter.CompileFile( GetScriptName() );
    {$IFDEF DEBUG}
    DebugPrint;
    {$ENDIF}
    Interpreter.Run;
    if not Response.Sent then
    begin
      Response.ContentStream := ContentStream;
      ContentStream.Position := 0;
      Response.ContentLength := ContentStream.Size;
      Response.SendResponse;
    end;
  finally
    Interpreter.Free;
  end;
end;

function TCanCGIApplication.MapPath(const LogicalPath: string): string;
begin
  Result := Request.MapPath( LogicalPath );
  //Result := Request.TranslateURI( LogicalPath );
end;

procedure TCanCGIApplication.Realm(const ARealm: string);
begin
  Response.Realm := ARealm;
end;

procedure TCanCGIApplication.Redirect(const URI: string);

  function IsValidURL(const URL: string): Boolean;
  begin
    Result := Pos('://', URL) > 0;
  end;
  {
  function IsValidURL(const URL: WideString): Boolean;
  var
    ResultStr: array[0..1023] of WideChar;
    ResultLen: Cardinal;
  begin
    FillChar( ResultStr, SizeOf(ResultStr), 0 );
    Result := CoInternetParseUrl(PWideChar(URL), PARSE_SCHEMA, 0, ResultStr,
      SizeOf(ResultStr), ResultLen, 0 ) <> S_FALSE;
  end;
  }
var
  URL, Host: string;
begin
  ContentStream.Clear;
  if IsValidURL( URI ) then URL := URI
  else begin
    Host := Request.Host;
    if Host = '' then Host := 'localhost';
    URL := 'http://' + Host;
    if URI[1] <> '/' then URL := URL + '/';
    URL := URL + URI;
  end;
  Response.SendRedirect( URL );
end;

procedure TCanCGIApplication.Run;
begin
  if IsConsole then
  begin
    Rewrite(Output);
    Reset(Input);
  end;
  try
    FRequest := TCGIScriptRequest.Create;
    try
      FResponse := TCGIScriptResponse.Create(FRequest);
      try
        HandleRequest;
      finally
        FResponse.Free;
      end;
    finally
      FRequest.Free;
    end;
  except
    HandleServerException(ExceptObject);
  end;
end;

{ InitApplication }
procedure InitApplication;
begin
  Application := TCanCGIApplication.Create;
end;

procedure TCanCGIApplication.SendStream(AStream: TStream);
begin
  ContentStream.Clear;
  ContentStream.CopyFrom( AStream, AStream.Size );
  //Response.SendStream( AStream );
end;

procedure TCanCGIApplication.StatusCode(const Value: Integer);
begin
  Response.StatusCode := Value;
end;

procedure TCanCGIApplication.Write(Data: Variant);
begin
  if Assigned(ContentStream) then
    ContentStream.WriteString(Data);
end;

function TCanCGIApplication.WriteString(const AString: string): Boolean;
begin
  if Assigned(ContentStream) then
    Result := ContentStream.WriteString(AString)
  else Result := False;
end;

procedure TCanCGIApplication.WWWAuthenticate(const Authenticate: string);
begin
  Response.WWWAuthenticate := Authenticate;
end;

{ TContentStream }

function TContentStream.WriteString(const AString: string): Boolean;
begin
  if AString <> '' then
    Result := Write(Pointer(AString)^, Length(AString)) = Length(AString)
  else Result := False;
end;

{ TCGIScriptRequest }

function TCGIScriptRequest.MapPath(const LogicalPath: string): string;
var
  HostPart, Path: string;
begin
  Result := '';

  if LogicalPath = '' then
    raise Exception.Create('Logical Path cannot be empty.');

  if LogicalPath[1] <> '/' then
  begin
    Path := ExtractFilePath( PathTranslated );
    Result := Path + UnixPathToDosPath( LogicalPath );
  end
  else begin
    HostPart := LowerCase( GetHostPart( Host ) );
    Result := WebServerMapPath( HostPart, LogicalPath );
  end;
  
end;

function TCGIScriptRequest.TranslateURI(const URI: string): string;
var
  PathInfo: string;
  PathInfoPos: Integer;
  PathTranslated: string;
  {
  OutBuf: array[0..260-1] of Char;
  InOutByte: Cardinal;
  }

begin
  {
  PathIsURL(PChar(URI));
  InOutByte := SizeOf(OutBuf);
  URLGetPart(PChar(URI), OutBuf, InOutByte, 0, 0);
  }
  Result := URI;
  PathTranslated := GetStringVariable(5);
  if PathTranslated <> '' then
  begin
    PathInfo := UnixPathToDosPath( GetStringVariable(4) );
    PathInfoPos := Length(PathTranslated) - Length(PathInfo);
    if LowerCase( Copy(PathTranslated, PathInfoPos + 1, Length(PathInfo)) ) = LowerCase( PathInfo ) then
      Result := Copy(PathTranslated, 1, PathInfoPos) + UnixPathToDosPath( URI );
    Result := LowerCase( Result );
  end;
end;

initialization
  //InitApplication;

finalization

end.
