unit Compiler;

interface

uses SysUtils, Classes, ScriptParser, DBTables, Variants;

type
  TCompiler = class(TObject)
  private
    FParser: TCustomParser;
  protected

  public
    constructor Create(Stream: TStream);
    destructor Destroy; override;
    procedure Compile;
    procedure CompileIdent;
    procedure CompileExecute;
    procedure CompileInclude;
    function Evaluate: Variant;
    procedure PrintException(const Text: string);
    function GetNextToken: Word;
    function GetIdentCode(const IdentText: string): Integer;
    property Parser: TCustomParser read FParser;
  end;

{ WriteClient }
function WriteClient(var Buffer; Count: Integer): Integer;

{ WriteString }
function WriteString(const AString: string): Boolean;

implementation

const
  MAX_WORD = 7;
  SpecialWords: array[0..MAX_WORD-1] of string = (
    'echo', 'include', 'date', 'time', 'database', 'table', 'execute'
  );

function WriteClient(var Buffer; Count: Integer): Integer;
begin
  Result := FileWrite(TTextRec(Output).Handle, Buffer, Count);
end;

function WriteString(const AString: string): Boolean;
begin
  if AString <> '' then
    Result := WriteClient(Pointer(AString)^, Length(AString)) = Length(AString)
  else Result := False;
end;

{ TCompiler }

procedure TCompiler.Compile;
var
  Token: Word;
begin
  while True do
  begin
    Token := GetNextToken;
    case Token of
      idEOF: Break;
      idHtml: WriteString( Parser.TokenString );
      idIdent: CompileIdent;
    end;
  end;
  //WriteLn( Parser.Line );
end;

procedure TCompiler.CompileExecute;
var
  Token: Word;
  Query: TQuery;
  DatabaseName, QueryText, Template: string;
  //Index: Integer;
  Stream: TMemoryStream;
  TemplateParser: TCustomParser;
  //FieldNames: TStringList;
  TemplateText: string;
  //FormatText: string;
  //Fields: Variant;
  //Fields: array of string;
begin
  Token := GetNextToken;
  if Token = idString then
  begin
    DatabaseName := Parser.TokenString;
    Token := GetNextToken;
    if Token = idComma then
    begin
      Token := GetNextToken;
      if Token = idString then
      begin
        QueryText := Parser.TokenString;
        Token := GetNextToken;
        if Token = idComma then
        begin
          Token := GetNextToken;
          if Token = idString then
          begin
            Template := Parser.TokenString;
            Stream := TMemoryStream.Create;
            try
              Stream.LoadFromFile( Template );
              Query := TQuery.Create( nil );
              try
                Query.DatabaseName := DatabaseName;
                Query.SQL.Clear;
                Query.SQL.Add( QueryText );
                Query.Open;
                Query.First;
                while not Query.Eof do begin
                  TemplateParser := TCustomParser.Create( Stream );
                  try
                    TemplateText := '';
                    while True do begin
                      Token := TemplateParser.NextToken;
                      case Token of
                        idEOF: Break;
                        idHtml: TemplateText := TemplateText + TemplateParser.TokenString;
                        idIdent:
                          begin
                            try
                              TemplateText := TemplateText +
                                Query.FieldByName(TemplateParser.TokenString).AsString;
                            except
                              TemplateText := TemplateText + 'Exception.';
                            end;
                          end;
                      end;
                    end;
                    WriteString( TemplateText );
                    WriteString( #13#10 );
                  finally
                    TemplateParser.Free;
                  end;

                  {
                  for Index := 0 to FieldNames.Count - 1 do begin
                    Fields[Index] := Query.FieldByName(FieldNames[Index]).AsString;
                    TemplateText := Format( FormatText, Fields );
                    Write( TemplateText );
                  end;
                  }
                  //WriteLn;
                  Query.Next;
                end;
                Query.Close;
              finally
                Query.Free;
              end;
              (*
              TemplateParser := TCustomParser.Create( Stream );
              try
                FieldNames := TStringList.Create;
                try
                  FormatText := '';
                  while True do begin
                    Token := TemplateParser.NextToken;
                    case Token of
                      idEOF: Break;
                      idHtml: FormatText := FormatText + TemplateParser.TokenString;
                      idIdent:
                        begin
                          FieldNames.Add( TemplateParser.TokenString );
                          FormatText := FormatText + '%s';
                        end;
                    end;
                  end;
                  Query := TQuery.Create( nil );
                  try
                    Query.DatabaseName := DatabaseName;
                    Query.SQL.Clear;
                    Query.SQL.Add( QueryText );
                    Query.Open;
                    Query.First;
                    TemplateText := '';
                    SetLength( Fields, FieldNames.Count );
                    //Fields := VarArrayCreate( [0, FieldNames.Count - 1], vtString );
                    while not Query.Eof do begin
                      {
                      for Index := 0 to FieldNames.Count - 1 do begin
                        Fields[Index] := Query.FieldByName(FieldNames[Index]).AsString;
                        TemplateText := Format( FormatText, Fields );
                        Write( TemplateText );
                      end;
                      }
                      WriteLn;
                      Query.Next;
                    end;
                    Query.Close;
                  finally
                    Query.Free;
                  end;
                finally
                  FieldNames.Free;
                end;

              finally
                TemplateParser.Free;
              end;
              *)
            finally
              Stream.Free;
            end;
          end
          else PrintException( 'Template name expected.' );
        end
        else PrintException( 'Comma expected.' );
      end
      else PrintException( 'Query text expected.' );
    end
    else PrintException( 'Comma expected.' );
  end
  else PrintException( 'Database name expected.' );
end;

procedure TCompiler.CompileIdent;
var
  Token: Word;
begin
  if SameText( LowerCase( Parser.TokenString ), 'echo' ) then
  begin
    Token := GetNextToken;
    if Token = idLParenthes then
    begin
      repeat
        Token := GetNextToken;
        case Token of
          idEOF: Break;
          idString,
          idInteger,
          idFloat: WriteString( Parser.TokenString() );
          idIdent: CompileIdent;
          idComma: Continue;
        end;
      until Token = idRParenthes;
      if Token <> idRParenthes then PrintException( 'Right Parenthes expected.' );
    end
    else PrintException( 'Left Parenthes expected.' );
  end
  else if SameText( LowerCase( Parser.TokenString ), 'include' ) then
  begin
    Token := GetNextToken;
    if Token = idLParenthes then
    begin
      repeat
        Token := GetNextToken;
        case Token of
          idEOF: Break;
          idString: CompileInclude;
        end;
      until Token = idRParenthes;
      if Token <> idRParenthes then PrintException( 'Right Parenthes expected.' );
    end
    else PrintException( 'Left Parenthes expected.' );
  end
  else if SameText( LowerCase( Parser.TokenString ), 'now' ) then
  begin
    WriteString( DateTimeToStr( Now() ) );
  end
  else if SameText( LowerCase( Parser.TokenString ), 'date' ) then
  begin
    WriteString( DateToStr( Date() ) );
  end
  else if SameText( LowerCase( Parser.TokenString ), 'time' ) then
  begin
    WriteString( TimeToStr( Time() ) );
  end
  else if SameText( LowerCase( Parser.TokenString ), 'execute' ) then
  begin
    Token := GetNextToken;
    if Token = idLParenthes then
    begin
      CompileExecute;
      Token := GetNextToken;
      if Token <> idRParenthes then PrintException( 'Right Parenthes expected.' );
    end
    else PrintException( 'Left Parenthes expected.' );
  end;
end;

procedure TCompiler.CompileInclude;
var
  IncludeFile: string;
  Stream: TMemoryStream;
  Interpreter: TCompiler;
begin
  IncludeFile := Parser.TokenString;
  Stream := TMemoryStream.Create;
  try
    Stream.LoadFromFile( IncludeFile );
    Interpreter := TCompiler.Create( Stream );
    try
      Interpreter.Compile;
    finally
      Interpreter.Free;
    end;
  finally
    Stream.Free;
  end;
end;

constructor TCompiler.Create(Stream: TStream);
begin
  FParser := TCustomParser.Create( Stream );
end;

destructor TCompiler.Destroy;
begin
  FParser.Free;
  inherited Destroy;
end;

function TCompiler.Evaluate: Variant;
begin
  Result := Unassigned;
end;

function TCompiler.GetIdentCode(const IdentText: string): Integer;
begin
  Result := -1;
end;

function TCompiler.GetNextToken: Word;
begin
  repeat
    Result := Parser.NextToken;
  until Result <> idComment;
end;

procedure TCompiler.PrintException(const Text: string);
begin
  WriteString( Format( 'Line: %d, Exception:%s', [Parser.SourcePos, Text] ) );
end;

end.
