
{.$DEFINE DEBUG}

unit ScriptFuncs;

interface

uses SysUtils, Classes, Contnrs, Math, Variants, ScriptParser, SysObjList,
  SysUnit, EncdDecd, ScriptVariants;

type
  TScriptParamList = class;

  { TScriptParam }
  TScriptParam = class(TObject)
  private
    FOwnerList: TScriptParamList;
    FName: string;
    FValue: Variant;
    FParamType: Integer;
    FOptions: Cardinal;
    FObjectValue: TObject;
    FVarParam: TScriptVariant;
    procedure SetName(const Value: string);
    procedure SetValue(const Value: Variant);
  protected

  public
    constructor Create(AOwnerList: TScriptParamList);
    destructor Destroy; override;
    property Name: string read FName write SetName;
    property Value: Variant read FValue write SetValue;
    property ParamType: Integer read FParamType write FParamType;
    property ObjectValue: TObject read FObjectValue write FObjectValue;
    property VarParam: TScriptVariant read FVarParam write FVarParam;
    property Options: Cardinal read FOptions write FOptions;
  end;

  { TScriptParamList }
  TScriptParamList = class(TObjectList)
  protected
    function GetItem(Index: Integer): TScriptParam;
    procedure SetItem(Index: Integer; AParam: TScriptParam);
  public
    function Add(AParam: TScriptParam): Integer;
    function AddParam: TScriptParam;
    function IndexOf(AParam: TScriptParam): Integer;
    function IndexOfName(const AName: string): Integer;
    function IsDefined(const AName: string): Boolean;
    function Remove(AParam: TScriptParam): Integer;
    procedure Insert(Index: Integer; AParam: TScriptParam);
    property Items[Index: Integer]: TScriptParam read GetItem write SetItem; default;
  end;

  TScriptFunctionList = class;
  TScriptFunctionClass = class of TScriptFunction;

  { TScriptFunction }
  TScriptFunction = class(TObject)
  private
    FOwnerList: TScriptFunctionList;
    FName: string;
    FResult: Variant;
    FResultType: Integer;
    FParams: TScriptParamList;
    FObjectResult: TObject;
  protected
    procedure SetName(const Value: string);
    function GetParamCount: Integer; virtual;
  public
    constructor Create(AOwnerList: TScriptFunctionList); virtual;
    destructor Destroy; override;
    function Execute: Variant; virtual;
    property Name: string read FName write SetName;
    property Params: TScriptParamList read FParams;
    property Result: Variant read FResult;
    property ResultType: Integer read FResultType write FResultType;
    property ObjectResult: TObject read FObjectResult write FObjectResult;
    property ParamCount: Integer read GetParamCount;
  end;

  { TScriptFunctionList }
  TScriptFunctionList = class(TObjectList)
  protected
    function GetItem(Index: Integer): TScriptFunction;
    procedure SetItem(Index: Integer; AFunction: TScriptFunction);
    function GetFunction(Index: string): TScriptFunction;
    procedure SetFunction(Index: string; AFunction: TScriptFunction);
  public
    function IndexOf(AFunction: TScriptFunction): Integer;
    function IndexOfName(const AName: string): Integer;
    function FunctionByName(const AName: string): Integer;
    function IsDefined(const AName: string): Boolean;
    function AddFunction(FuncClass: TScriptFunctionClass): Integer;
    function Add(AFunction: TScriptFunction): Integer;
    function Remove(AFunction: TScriptFunction): Integer;
    procedure Insert(Index: Integer; AFunction: TScriptFunction);
    property Items[Index: Integer]: TScriptFunction read GetItem write SetItem; default;
    property Functions[Index: string]: TScriptFunction read GetFunction write SetFunction;
  end;

  { TGetServerVariableFunc }
  TGetServerVariableFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TGetTimeFunc }
  TGetTimeFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TIntToStrFunc }
  TIntToStrFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStrToIntFunc }
  TStrToIntFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TSqrtFunc }
  TSqrtFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TPrintFunc }
  TPrintFunc = class(TScriptFunction)
  protected
    function GetParamCount: Integer; override;
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TPrintLnFunc }
  TPrintLnFunc = class(TScriptFunction)
  protected
    function GetParamCount: Integer; override;
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TOpenFileFunc }
  TOpenFileFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TWriteFileFunc }
  TWriteFileFunc = class(TScriptFunction)
  protected
    function GetParamCount: Integer; override;
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TCloseFileFunc }
  TCloseFileFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TCreateFileFunc }
  TCreateFileFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TLengthFunc }
  TLengthFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TTimeToStrFunc }
  TTimeToStrFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TLowerCaseFunc }
  TLowerCaseFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TUpperCaseFunc }
  TUpperCaseFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TGetQueryStringFunc }
  TGetQueryStringFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TPosFunc }
  TPosFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TCopyFunc }
  TCopyFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TFreeFunc }
  TFreeFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TIncFunc }
  TIncFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TFloatToStrFunc }
  TFloatToStrFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TFloatToStrFFunc }
  TFloatToStrFFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TDateToStrFunc }
  TDateToStrFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TWriteFunc }
  TWriteFunc = class(TScriptFunction)
  protected
    function GetParamCount: Integer; override;
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TWriteLnFunc }
  TWriteLnFunc = class(TScriptFunction)
  protected
    function GetParamCount: Integer; override;
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TServerMapPathFunc }
  TServerMapPathFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TRequestFormFunc }
  TRequestFormFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TRedirectFunc }
  TRedirectFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TSetCookieFunc }
  TSetCookieFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TCookieFunc }
  TCookieFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TGetDateFunc }
  TGetDateFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TGetNowFunc }
  TGetNowFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TExpiresFunc }
  TExpiresFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TMakeSessionIDFunc }
  TMakeSessionIDFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TDateTimeToStrFunc }
  TDateTimeToStrFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TCharAtFunc }
  TCharAtFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TContentTypeFunc }
  TContentTypeFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TSendStreamFunc }
  TSendStreamFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TFileStreamCreateFunc }
  TFileStreamCreateFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TWWWAuthenticateFunc }
  TWWWAuthenticateFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TRealmFunc }
  TRealmFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStatusCodeFunc }
  TStatusCodeFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TEncode64Func }
  TEncode64Func = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TDecode64Func }
  TDecode64Func = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TSendResponseFunc }
  TSendResponseFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TFileExistsFunc }
  TFileExistsFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TGetEnvironmentStringsFunc }
  TGetEnvironmentStringsFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

{ RegisterFunctions }
procedure RegisterFunctions(Funcs: array of TScriptFunctionClass);

var
  ScriptFunctionList: TScriptFunctionList;

implementation

uses Windows, CanApp;//, HTTPApp, CGIHTTP;

{ InitScriptFunctionList }
procedure InitScriptFunctionList;
begin
  ScriptFunctionList := TScriptFunctionList.Create;
end;

procedure DoneScriptFunctionList;
begin
  ScriptFunctionList.Free;
end;

{ RegisterFunctions }
procedure RegisterFunctions(Funcs: array of TScriptFunctionClass);
var
  I: Integer;
begin
  for I := Low(Funcs) to High(Funcs) do
    ScriptFunctionList.AddFunction(Funcs[I]);
end;

procedure RegisterSystemFunctions(Funcs: array of TScriptFunctionClass);
var
  I: Integer;
begin
  for I := Low(Funcs) to High(Funcs) do
    ScriptFunctionList.AddFunction(Funcs[I]);
end;

{ RegisterScriptFunctions }
procedure RegisterScriptFunctions;
begin
  RegisterSystemFunctions( [TGetServerVariableFunc, TGetTimeFunc,
    TIntToStrFunc, TStrToIntFunc, TSqrtFunc, TPrintFunc, TPrintLnFunc,
    TOpenFileFunc, TWriteFileFunc, TCloseFileFunc, TCreateFileFunc,
    TLengthFunc, TTimeToStrFunc, TLowerCaseFunc, TUpperCaseFunc,
    TGetQueryStringFunc, TPosFunc, TCopyFunc, TFreeFunc, TFloatToStrFunc,
    TFloatToStrFFunc, TDateToStrFunc, TWriteFunc, TWriteLnFunc,
    TServerMapPathFunc, TRequestFormFunc, TRedirectFunc, TSetCookieFunc,
    TCookieFunc, TGetDateFunc, TGetNowFunc, TExpiresFunc, TMakeSessionIDFunc,
    TDateTimeToStrFunc, TCharAtFunc, TContentTypeFunc, TSendStreamFunc,
    TFileStreamCreateFunc, TWWWAuthenticateFunc, TRealmFunc, TStatusCodeFunc,
    TEncode64Func, TDecode64Func, TSendResponseFunc, TFileExistsFunc,
    TGetEnvironmentStringsFunc] );
end;

{ TScriptParam }

constructor TScriptParam.Create(AOwnerList: TScriptParamList);
begin
  inherited Create;
  FOwnerList := AOwnerList;
end;

destructor TScriptParam.Destroy;
begin
  FValue := Unassigned;
  inherited Destroy;
end;

procedure TScriptParam.SetName(const Value: string);
begin
  if not FOwnerList.IsDefined(Value) then
    FName := Value
  else raise Exception.CreateFmt('Variable %s redefined.', [Value]);
end;

procedure TScriptParam.SetValue(const Value: Variant);
begin
  FValue := Value;
end;

{ TScriptFunction }

constructor TScriptFunction.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create;
  FOwnerList := AOwnerList;
  FParams := TScriptParamList.Create;
end;

destructor TScriptFunction.Destroy;
begin
  FParams.Free;
  FResult := Unassigned;
  inherited Destroy;
end;

function TScriptFunction.Execute: Variant;
begin
  // abstract method
  Result := Unassigned;
end;

function TScriptFunction.GetParamCount: Integer;
begin
  Result := Params.Count;
end;

procedure TScriptFunction.SetName(const Value: string);
begin
  if not FOwnerList.IsDefined(Value) then
    FName := Value
  else raise Exception.CreateFmt('Function %s redefined.', [Value]);
end;

{ TScriptFunctionList }

function TScriptFunctionList.Add(AFunction: TScriptFunction): Integer;
begin
  Result := inherited Add(AFunction);
end;

function TScriptFunctionList.AddFunction(FuncClass: TScriptFunctionClass): Integer;
begin
  Result := inherited Add(FuncClass.Create(Self));
end;

function TScriptFunctionList.FunctionByName(const AName: string): Integer;
var
  I: Integer;
  Item: TScriptFunction;
begin
  Result := -1;
  for I := 0 to Self.Count - 1 do begin
    Item := TScriptFunction(Self.Items[I]);
    if SameText(Item.Name, AName) then
    begin
      Result := I;
      Break;
    end;
  end;
end;

function TScriptFunctionList.GetFunction(Index: string): TScriptFunction;
begin
  Result := GetItem(IndexOfName(Index));
end;

function TScriptFunctionList.GetItem(Index: Integer): TScriptFunction;
begin
  Result := TScriptFunction(inherited Items[Index]);
end;

function TScriptFunctionList.IndexOf(AFunction: TScriptFunction): Integer;
begin
  Result := inherited IndexOf(AFunction);
end;

function TScriptFunctionList.IndexOfName(const AName: string): Integer;
var
  I: Integer;
  Item: TScriptFunction;
begin
  Result := -1;
  for I := 0 to Self.Count - 1 do begin
    Item := Self.Items[I];
    if SameText(Item.Name, AName) then
    begin
      Result := I;
      Break;
    end;
  end;
end;

procedure TScriptFunctionList.Insert(Index: Integer;
  AFunction: TScriptFunction);
begin
  inherited Insert(Index, AFunction);
end;

function TScriptFunctionList.IsDefined(const AName: string): Boolean;
begin
  Result := Self.IndexOfName(AName) > -1;
end;

function TScriptFunctionList.Remove(AFunction: TScriptFunction): Integer;
begin
  Result := inherited Remove(AFunction);
end;

procedure TScriptFunctionList.SetFunction(Index: string;
  AFunction: TScriptFunction);
begin
  SetItem(IndexOfName(Index), AFunction);
end;

procedure TScriptFunctionList.SetItem(Index: Integer;
  AFunction: TScriptFunction);
begin
  inherited Items[Index] := AFunction;
end;

{ TScriptParamList }

function TScriptParamList.Add(AParam: TScriptParam): Integer;
begin
  Result := inherited Add(AParam);
end;

function TScriptParamList.AddParam: TScriptParam;
begin
  Result := TScriptParam.Create(Self);
  Add(Result);
end;

function TScriptParamList.GetItem(Index: Integer): TScriptParam;
begin
  Result := TScriptParam(inherited Items[Index]);
end;

function TScriptParamList.IndexOf(AParam: TScriptParam): Integer;
begin
  Result := inherited IndexOf(AParam);
end;

function TScriptParamList.IndexOfName(const AName: string): Integer;
var
  I: Integer;
  Item: TScriptParam;
begin
  Result := -1;
  for I := 0 to Self.Count - 1 do begin
    Item := TScriptParam(Self.Items[I]);
    if SameText( Item.Name, AName ) then
    begin
      Result := I;
      Break;
    end;
  end;
end;

procedure TScriptParamList.Insert(Index: Integer; AParam: TScriptParam);
begin
  inherited Insert(Index, AParam);
end;

function TScriptParamList.IsDefined(const AName: string): Boolean;
begin
  Result := Self.IndexOfName(AName) > -1;
end;

function TScriptParamList.Remove(AParam: TScriptParam): Integer;
begin
  Result := inherited Remove(AParam);
end;

procedure TScriptParamList.SetItem(Index: Integer;
  AParam: TScriptParam);
begin
  inherited Items[Index] := AParam;
end;

{ TGetServerVariableFunc }

constructor TGetServerVariableFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'GetServerVariable';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@VarName';
    ParamType := idString;
  end;
end;

function TGetServerVariableFunc.Execute: Variant;
var
  DefText: string;
begin
  DefText := Params[0].Value;
  Result := GetEnvironmentVariable(DefText);
  Params[0].Value := Unassigned;
end;

{ TGetTimeFunc }

constructor TGetTimeFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'GetTime';
  ResultType := idFloat;
end;

function TGetTimeFunc.Execute: Variant;
begin
  Result := SysUtils.Time();
end;

{ TIntToStrFunc }

constructor TIntToStrFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'IntToStr';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Int';
    ParamType := idInteger;
  end;
end;

function TIntToStrFunc.Execute: Variant;
begin
  Result := SysUtils.IntToStr(Params[0].Value);
end;

{ TStrToIntFunc }

constructor TStrToIntFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StrToInt';
  ResultType := idInteger;
  with Params.AddParam do begin
    Name := '@StrVal';
    ParamType := idString;
  end;
end;

function TStrToIntFunc.Execute: Variant;
begin
  Result := SysUtils.StrToInt(Params[0].Value);
end;

{ TSqrtFunc }

constructor TSqrtFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Sqrt';
  ResultType := idFloat;
  with Params.AddParam do begin
    Name := '@Int';
    ParamType := idFloat;
  end;
end;

function TSqrtFunc.Execute: Variant;
begin
  Result := Sqrt(Params[0].Value);
end;

{ TPrintFunc }

constructor TPrintFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Print';
  ResultType := idUnknown;
  // OpenParam: Parametreler Run-Time da olusturuluyor.
end;

function TPrintFunc.Execute: Variant;
var
  Text: string;
  Index, Count: Integer;
begin
  Result := Unassigned;
  Count := Params.Count;
  for Index := 0 to Count - 1 do
  begin
    Text := Params[Index].Value;
    Application.WriteString(Text);
  end;
  for Index := 0 to Count - 1 do
    Params[Index].Value := Unassigned;
end;

function TPrintFunc.GetParamCount: Integer;
begin
  Result := -1; // OpenParam;
end;

{ TOpenFileFunc }

constructor TOpenFileFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'OpenFile';
  ResultType := idInteger;
  with Params.AddParam do begin
    Name := '@FileName';
    ParamType := idString;
  end;
  with Params.AddParam do begin
    Name := '@FileMode';
    ParamType := idInteger;
  end;
end;

function TOpenFileFunc.Execute: Variant;
var
  FileName: string;
  FileMode: LongWord;
  Handle: THandle;
begin
  FileName := Params[0].Value;
  FileMode := Params[1].Value;
  Handle := SysUtils.FileOpen( FileName, FileMode );
  if Handle <> INVALID_HANDLE_VALUE then
    SystemHandleList.Add( Handle );
  Result := Handle;
  Params[0].Value := Unassigned;
  Params[1].Value := Unassigned;
end;

{ TWriteFileFunc }

constructor TWriteFileFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'WriteFile';
  ResultType := idInteger;
  {
  with Params.AddParam do begin
    Name := '@FileHandle';
    ParamType := idInteger;
  end;
  with Params.AddParam do begin
    Name := '@Buffer';
    ParamType := idString;
  end;
  }
end;

function TWriteFileFunc.Execute: Variant;
var
  Text: string;
  Index, Count: Integer;
  Handle: THandle;
begin
  Result := 0;
  Count := Params.Count;
  Handle := Params[0].Value;
  for Index := 1 to Count - 1 do
  begin
    Text := Params[Index].Value;
    Result := Result + SysUtils.FileWrite( Handle, Pointer(Text)^, Length(Text) );
  end;
  for Index := 0 to Count - 1 do
    Params[Index].Value := Unassigned;
end;

function TWriteFileFunc.GetParamCount: Integer;
begin
  Result := -1; // OpenParam;
end;

{ TCloseFileFunc }

constructor TCloseFileFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'CloseFile';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@FileHandle';
    ParamType := idInteger;
  end;
end;

function TCloseFileFunc.Execute: Variant;
var
  FileHandle: Integer;
begin
  FileHandle := Params[0].Value;
  SystemHandleList.Remove(FileHandle);
  //SysUtils.FileClose( FileHandle );
  Params[0].Value := Unassigned;
end;

{ TCreateFileFunc }

constructor TCreateFileFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'CreateFile';
  ResultType := idInteger;
  with Params.AddParam do begin
    Name := '@FileName';
    ParamType := idString;
  end;
end;

function TCreateFileFunc.Execute: Variant;
var
  FileName: string;
  Handle: THandle;
begin
  FileName := Params[0].Value;
  Handle := SysUtils.FileCreate( FileName );
  if Handle <> INVALID_HANDLE_VALUE then
    SystemHandleList.Add( Handle );
  Result := Handle;
end;

{ TLengthFunc }

constructor TLengthFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Length';
  ResultType := idInteger;
  with Params.AddParam do begin
    Name := '@String';
    ParamType := idString;
  end;
end;

function TLengthFunc.Execute: Variant;
var
  Text: string;
begin
  Text := Params[0].Value;
  Result := Length( Text );
end;

{ TPrintLnFunc }

constructor TPrintLnFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'PrintLn';
  ResultType := idUnknown;
  // OpenParam: Parametreler Run-Time da olusturuluyor.
end;

function TPrintLnFunc.Execute: Variant;
var
  Text: string;
  Index, Count: Integer;
begin
  Result := Unassigned;
  Count := Params.Count;
  for Index := 0 to Count - 1 do
  begin
    Text := Params[Index].Value;
    Application.WriteString(Text);
  end;
  Application.WriteString( CRLF );
  for Index := 0 to Count - 1 do
    Params[Index].Value := Unassigned;
end;

function TPrintLnFunc.GetParamCount: Integer;
begin
  Result := -1; // OpenParam;
end;

{ TTimeToStrFunc }

constructor TTimeToStrFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'TimeToStr';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Time';
    ParamType := idFloat;
  end;
end;

function TTimeToStrFunc.Execute: Variant;
var
  Value: TDateTime;
begin
  Value := Params[0].Value;
  Result := TimeToStr( Value );
  Params[0].Value := UnAssigned;
end;

{ TLowerCaseFunc }

constructor TLowerCaseFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'LowerCase';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Str';
    ParamType := idString;
  end;
end;

function TLowerCaseFunc.Execute: Variant;
var
  Value: string;
begin
  Value := Params[0].Value;
  Result := LowerCase( Value );
  Params[0].Value := UnAssigned;
end;

{ TUpperCaseFunc }

constructor TUpperCaseFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'UpperCase';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Str';
    ParamType := idString;
  end;
end;

function TUpperCaseFunc.Execute: Variant;
var
  Value: string;
begin
  Value := Params[0].Value;
  Result := UpperCase( Value );
  Params[0].Value := UnAssigned;
end;

{ TGetQueryStringFunc }

constructor TGetQueryStringFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'GetQueryString';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Str';
    ParamType := idString;
  end;
end;

function TGetQueryStringFunc.Execute: Variant;
var
  Ident: string;
begin
  Ident := Params[0].Value;
  Ident := LowerCase(Ident);
  Result := Application.Request.QueryFields.Values[Ident];
  Params[0].Value := UnAssigned;
end;

{ TPosFunc }

constructor TPosFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Pos';
  ResultType := idInteger;
  with Params.AddParam do begin
    Name := '@SubStr';
    ParamType := idString;
  end;
  with Params.AddParam do begin
    Name := '@Str';
    ParamType := idString;
  end;
end;

function TPosFunc.Execute: Variant;
var
  SubStr, S: string;
begin
  SubStr := Params[0].Value;
  S := Params[1].Value;
  Result := System.Pos(SubStr, S);
  Params[0].Value := UnAssigned;
  Params[1].Value := UnAssigned;
end;

{ TCopyFunc }

constructor TCopyFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Copy';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@S';
    ParamType := idString;
  end;
  with Params.AddParam do begin
    Name := '@Index';
    ParamType := idInteger;
  end;
  with Params.AddParam do begin
    Name := '@Count';
    ParamType := idInteger;
  end;
end;

function TCopyFunc.Execute: Variant;
var
  S: string;
  Index, Count: Integer;
begin
  S := Params[0].Value;
  Index := Params[1].Value;
  Count := Params[2].Value;
  Result := System.Copy(S, Index, Count);
  Params[0].Value := UnAssigned;
  Params[1].Value := UnAssigned;
  Params[2].Value := UnAssigned;
end;

{ TFreeFunc }

constructor TFreeFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Free';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@Object';
    ParamType := idObject;
  end;
end;

function TFreeFunc.Execute: Variant;
var
  Obj: TObject;
begin
  Obj := Params[0].ObjectValue;
  if SystemObjectList.Remove(Obj) < 0 then
    WriteLn('Failed Free object.'); ///// Kaldirmayi unutma!
  Params[0].ObjectValue := nil;
  Params[0].Value := UnAssigned;
end;

{ TIncFunc }

constructor TIncFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Inc';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@Value';
    ParamType := idInteger;
    Options := idVariant;
  end;
end;

function TIncFunc.Execute: Variant;
begin

end;

{ TFloatToStrFunc }

constructor TFloatToStrFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'FloatToStr';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Value';
    ParamType := idFloat;
  end;
end;

function TFloatToStrFunc.Execute: Variant;
var
  Value: Extended;
begin
  Value := Params[0].Value;
  Result := FloatToStr(Value);
  Params[0].ObjectValue := nil;
  Params[0].Value := UnAssigned;
end;

{ TFloatToStrFFunc }

constructor TFloatToStrFFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'FloatToStrF';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Value';
    ParamType := idFloat;
  end;
  with Params.AddParam do begin
    Name := '@Format';
    ParamType := idInteger;
  end;
end;

function TFloatToStrFFunc.Execute: Variant;
var
  Value: Extended;
  Format: TFloatFormat;
begin
  Value := Params[0].Value;
  Format := Params[1].Value;
  Result := FloatToStrF(Value, Format, 15, 2);
  Params[0].Value := UnAssigned;
  Params[1].Value := UnAssigned;
end;

{ TDateToStrFunc }

constructor TDateToStrFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'DateToStr';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Date';
    ParamType := idFloat;
  end;
end;

function TDateToStrFunc.Execute: Variant;
var
  Value: TDateTime;
begin
  Value := Params[0].Value;
  Result := DateToStr( Value );
  Params[0].Value := UnAssigned;
end;

{ TWriteFunc }

constructor TWriteFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Write';
  ResultType := idUnknown;
  // OpenParam: Parametreler Run-Time da olusturuluyor.
end;

function TWriteFunc.Execute: Variant;
var
  Text: string;
  Index, Count: Integer;
begin
  Result := Unassigned;
  Count := Params.Count;
  for Index := 0 to Count - 1 do
  begin
    Text := Params[Index].Value;
    Application.WriteString(Text);
  end;
  for Index := 0 to Count - 1 do
    Params[Index].Value := Unassigned;
end;

function TWriteFunc.GetParamCount: Integer;
begin
  Result := -1; //OpenParam
end;

{ TWriteLnFunc }

constructor TWriteLnFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'WriteLn';
  ResultType := idUnknown;
  // OpenParam: Parametreler Run-Time da olusturuluyor.
end;

function TWriteLnFunc.Execute: Variant;
var
  Text: string;
  Index, Count: Integer;
begin
  Result := Unassigned;
  Count := Params.Count;
  for Index := 0 to Count - 1 do
  begin
    Text := Params[Index].Value;
    Application.WriteString(Text);
  end;
  Application.WriteString( CRLF );
  for Index := 0 to Count - 1 do
    Params[Index].Value := Unassigned;
end;

function TWriteLnFunc.GetParamCount: Integer;
begin
  Result := -1; //OpenParam
end;

{ TServerMapPathFunc }

constructor TServerMapPathFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'ServerMapPath';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@LogicalPath';
    ParamType := idString;
  end;
end;

function TServerMapPathFunc.Execute: Variant;
var
  URI: string;
begin
  URI := Params[0].Value;
  //Result := Application.Request.MapPath( LogicalPath );
  //Result := Application.MapPath( LogicalPath );
  Result := Application.Request.TranslateURI( URI );
  //ServerMapPath( LogicalPath );
  Params[0].Value := UnAssigned;
end;

{ TRequestFormFunc }

constructor TRequestFormFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'RequestForm';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Value';
    ParamType := idString;
  end;
end;

function TRequestFormFunc.Execute: Variant;
var
  FormField: string;
begin
  FormField := Params[0].Value;
  FormField := LowerCase(FormField);
  Result := Application.Request.ContentFields.Values[FormField];
  Params[0].Value := UnAssigned;
end;

{ TRedirectFunc }

constructor TRedirectFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Redirect';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@Value';
    ParamType := idString;
  end;
end;

function TRedirectFunc.Execute: Variant;
var
  URI: string;
begin
  URI := Params[0].Value;
  Application.Redirect( URI );
  Params[0].Value := UnAssigned;
end;

{ TSetCookieFunc }

constructor TSetCookieFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'SetCookie';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@Name';
    ParamType := idString;
  end;
  with Params.AddParam do begin
    Name := '@Value';
    ParamType := idString;
  end;
  with Params.AddParam do begin
    Name := '@Expires';
    ParamType := idFloat;
  end;
  with Params.AddParam do begin
    Name := '@Path';
    ParamType := idString;
  end;
end;

function TSetCookieFunc.Execute: Variant;
var
  CookieName, CookieValue, CookiePath: string;
  CookieExpires: TDateTime;
begin
  CookieName := Params[0].Value;
  CookieValue := Params[1].Value;
  CookieExpires := Params[2].Value;
  CookiePath := Params[3].Value;
  with Application.Response.Cookies.Add do
  begin
    Name := CookieName;
    Value := CookieValue;
    Expires := CookieExpires;
    Path := CookiePath;
  end;
  Params[0].Value := UnAssigned;
  Params[1].Value := UnAssigned;
  Params[2].Value := UnAssigned;
  Params[3].Value := UnAssigned;
  Result := UnAssigned;
end;

{ TCookieFunc }

constructor TCookieFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Cookie';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Name';
    ParamType := idString;
  end;
end;

function TCookieFunc.Execute: Variant;
var
  CookieName, CookieValue: string;
  Index: Integer;
begin
  CookieName := Params[0].Value;
  Index := Application.Request.CookieFields.IndexOfName(CookieName);
  if Index < 0 then
    CookieValue := ''
  else
    CookieValue := Application.Request.CookieFields.ValueFromIndex[Index];
  Result := CookieValue;
  Params[0].Value := UnAssigned;
end;

{ TGetDateFunc }

constructor TGetDateFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'GetDate';
  ResultType := idFloat;
end;

function TGetDateFunc.Execute: Variant;
begin
  Result := SysUtils.Date();
end;

{ TGetNowFunc }

constructor TGetNowFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'GetNow';
  ResultType := idFloat;
end;

function TGetNowFunc.Execute: Variant;
begin
  Result := SysUtils.Now();
end;

{ TExpiresFunc }

constructor TExpiresFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Expires';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@Expires';
    ParamType := idFloat;
  end;
end;

function TExpiresFunc.Execute: Variant;
var
  Expires: TDateTime;
begin
  Expires := Params[0].Value;
  Application.Response.Expires := Expires;
  Result := UnAssigned;
  Params[0].Value := UnAssigned;
end;

{ TMakeSessionIDFunc }

constructor TMakeSessionIDFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'MakeSessionID';
  ResultType := idString;
end;

function TMakeSessionIDFunc.Execute: Variant;
begin
  Result := SysUnit.MakeSessionID( 24 );
end;

{ TDateTimeToStrFunc }

constructor TDateTimeToStrFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'DateTimeToStr';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@DateTime';
    ParamType := idFloat;
  end;
end;

function TDateTimeToStrFunc.Execute: Variant;
var
  Value: TDateTime;
begin
  Value := Params[0].Value;
  Result := DateTimeToStr( Value );
  Params[0].Value := UnAssigned;
end;

{ TCharAtFunc }

constructor TCharAtFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'CharAt';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@String';
    ParamType := idString;
  end;
  with Params.AddParam do begin
    Name := '@Index';
    ParamType := idInteger;
  end;
end;

function TCharAtFunc.Execute: Variant;
var
  Value: string;
  Index: Integer;
begin
  Value := Params[0].Value;
  Index := Params[1].Value;
  Result := Value[Index];
  Params[0].Value := UnAssigned;
  Params[1].Value := UnAssigned;
end;

{ TContentTypeFunc }

constructor TContentTypeFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'ContentType';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@ContentType';
    ParamType := idString;
  end;
end;

function TContentTypeFunc.Execute: Variant;
var
  ContentType: string;
begin
  ContentType := Params[0].Value;
  Application.ContentType( ContentType );
  Params[0].Value := UnAssigned;
end;


{ TSendStreamFunc }

constructor TSendStreamFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'SendStream';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@Stream';
    ParamType := idObject;
  end;
end;

function TSendStreamFunc.Execute: Variant;
var
  Stream: TStream;
begin
  Stream := Params[0].ObjectValue as TStream;
  Application.SendStream( Stream );
  Params[0].ObjectValue := nil;
end;

{ TFileStreamCreateFunc }

constructor TFileStreamCreateFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'FileStream_Create';
  ResultType := idObject;
  with Params.AddParam do begin
    Name := '@FileName';
    ParamType := idString;
  end;
end;

function TFileStreamCreateFunc.Execute: Variant;
var
  Stream: TFileStream;
  FileName: string;
begin
  FileName := Params[0].Value;
  Stream := TFileStream.Create( FileName, fmOpenRead );
  ObjectResult := Stream;
  SystemObjectList.Add( Stream );
  Params[0].Value := Unassigned;
end;

{ TWWWAuthenticateFunc }

constructor TWWWAuthenticateFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'WWWAuthenticate';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@Authenticate';
    ParamType := idString;
  end;
end;

function TWWWAuthenticateFunc.Execute: Variant;
var
  WWWAuthenticate: string;
begin
  WWWAuthenticate := Params[0].Value;
  Application.WWWAuthenticate( WWWAuthenticate );
  Params[0].Value := UnAssigned;
end;

{ TRealmFunc }

constructor TRealmFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Realm';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@Realm';
    ParamType := idString;
  end;
end;

function TRealmFunc.Execute: Variant;
var
  Realm: string;
begin
  Realm := Params[0].Value;
  Application.Realm( Realm );
  Params[0].Value := UnAssigned;
end;

{ TStatusCodeFunc }

constructor TStatusCodeFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StatusCode';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@Value';
    ParamType := idInteger;
  end;
end;

function TStatusCodeFunc.Execute: Variant;
var
  StatusCode: Integer;
begin
  StatusCode := Params[0].Value;
  Application.StatusCode( StatusCode );
  Params[0].Value := UnAssigned;
end;

{ TEncode64Func }

constructor TEncode64Func.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Encode64';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Value';
    ParamType := idString;
  end;
end;

function TEncode64Func.Execute: Variant;
var
  Value: string;
begin
  Value := Params[0].Value;
  Result := EncodeString( Value );
  Params[0].Value := UnAssigned;
end;

{ TDecode64Func }

constructor TDecode64Func.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'Decode64';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@Value';
    ParamType := idString;
  end;
end;

function TDecode64Func.Execute: Variant;
var
  Value: string;
begin
  Value := Params[0].Value;
  Result := DecodeString( Value );
  Params[0].Value := UnAssigned;
end;

{ TSendResponseFunc }

constructor TSendResponseFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'SendResponse';
  ResultType := idUnknown;
end;

function TSendResponseFunc.Execute: Variant;
begin
  Result := UnAssigned;
  Application.Response.SendResponse;
end;

{ TFileExistsFunc }

constructor TFileExistsFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'FileExists';
  ResultType := idBoolean;
  with Params.AddParam do begin
    Name := '@Value';
    ParamType := idString;
  end;
end;

function TFileExistsFunc.Execute: Variant;
var
  Value: string;
begin
  Value := Params[0].Value;
  Result := FileExists( Value );
  Params[0].Value := UnAssigned;
end;

{ TGetEnvironmentStringsFunc }

constructor TGetEnvironmentStringsFunc.Create(
  AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'GetEnvironmentStrings';
  ResultType := idObject;
end;

function TGetEnvironmentStringsFunc.Execute: Variant;
var
  Strings: TStringList;
  Value: string;
  Environment, Data, Save: PChar;
begin
  Result := Unassigned;
  Strings := TStringList.Create;
  try
    Environment := GetEnvironmentStrings();
    try
      Data := Environment;
      while Data^ <> #0 do begin
        Save := Data;
        while Data^ <> #0 do Inc(Data);
        SetString(Value, Save, Data - Save);
        Inc(Data);
        if Data^ = #0 then Break;
        if Value <> '' then Strings.Add(Value);
      end;
    finally
      FreeEnvironmentStrings(Environment);
    end;
    ObjectResult := Strings;
    SystemObjectList.Add(Strings);
  except
    Strings.Free;
  end;
end;

initialization
  InitScriptFunctionList;
  RegisterScriptFunctions;

finalization
  DoneScriptFunctionList;

end.
