unit ScriptResponse;

interface

uses SysUtils, Classes;

type
  TResponse = class(TObject)
  private
    FStream: TMemoryStream;
    FBuffer: Boolean;
    FContentType: string;
    FStatus: string;
    FHeader: TStringList;
  protected

  public
    constructor Create;
    destructor Destroy; override;
    procedure AddHeader(const HeaderName, HeaderValue: string);
    procedure Flush;
    procedure Write(Data: Variant);
    procedure WriteString(const Text: string);
    procedure WriteClient(var Buffer; Count: Integer);
    procedure Clear;
    procedure Redirect(const URL: string);
    property Stream: TMemoryStream read FStream;
    property Buffer: Boolean read FBuffer write FBuffer;
    property ContentType: string read FContentType write FContentType;
    property Status: string read FStatus write FStatus;
    property Header: TStringList read FHeader write FHeader;
  end;

implementation

{ TResponse }

procedure TResponse.AddHeader(const HeaderName, HeaderValue: string);
begin
  FHeader.Add(HeaderName + '=' + HeaderValue);
end;

procedure TResponse.Clear;
begin
  Stream.Clear;
  Header.Clear;
end;

constructor TResponse.Create;
begin
  inherited Create;
  FStream := TMemoryStream.Create;
  FHeader := TStringList.Create;
end;

destructor TResponse.Destroy;
begin
  FHeader.Free;
  FStream.Free;
  inherited Destroy;
end;

procedure TResponse.Flush;
var
  Header: string;
begin
  if Status = '' then
    Status := '200 OK';
  if ContentType = '' then
    ContentType := 'text/html';
end;

procedure TResponse.Redirect(const URL: string);
begin

end;

procedure TResponse.Write(Data: Variant);
begin

end;

procedure TResponse.WriteClient(var Buffer; Count: Integer);
begin
  FileWrite(TTextRec(Output).Handle, Buffer, Count);
end;

procedure TResponse.WriteString(const Text: string);
begin
  if Buffer then
    Stream.WriteBuffer( PChar(Text)^, Length(Text) )
  else
    WriteClient( PChar(Text)^, Length(Text) );
end;

end.
