unit ScriptVariants;

interface

uses SysUtils, Classes, Contnrs, Variants, ScriptParser;

type
  TScriptVariantList = class;
  TScriptVariantClass = class of TScriptVariant;

  TScriptVariant = class(TObject)
  private
    FOwnerList: TScriptVariantList;
    FName: string;
    FValue: Variant;
    FObjectValue: TObject;
    FVarType: Integer;
  protected
    function GetValue: Variant; virtual;
    procedure SetValue(AValue: Variant); virtual;
    procedure SetName(const AValue: string);
  public
    constructor Create(AOwnerList: TScriptVariantList); virtual;
    destructor Destroy; override;
    property Name: string read FName write SetName;
    property Value: Variant read GetValue write SetValue;
    property VarType: Integer read FVarType write FVarType;
    property ObjectValue: TObject read FObjectValue write FObjectValue;
  end;

  TScriptVariantList = class(TObjectList)
  protected
    function GetItem(Index: Integer): TScriptVariant;
    procedure SetItem(Index: Integer; AVariant: TScriptVariant);
  public
    function IndexOfName(const AName: string): Integer;
    function IsDefined(const Value: string): Boolean;
    function GetVariant(const Value: string): TScriptVariant;
    function Add(AVariant: TScriptVariant): Integer;
    function AddVar: TScriptVariant;
    function CreateVar(const AName: string; AValue: Variant; AVarType: Integer): Integer;
    function AddVariant(VarClass: TScriptVariantClass): Integer;
    property Items[Index: Integer]: TScriptVariant read GetItem write SetItem; default;
  end;

  { TTrueVar }
  TTrueVar = class(TScriptVariant)
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

  { TFalseVar }
  TFalseVar = class(TScriptVariant)
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

  { TMaxIntVar }
  TMaxIntVar = class(TScriptVariant)
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

  { TTitleVar }
  TTitleVar = class(TScriptVariant)
  protected
    function GetValue: Variant; override;
    procedure SetValue(AValue: Variant); override;
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

  { TRemoteAddrVar }
  TRemoteAddrVar = class(TScriptVariant)
  protected
    function GetValue: Variant; override;
    procedure SetValue(AValue: Variant); override;
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

  { TPathInfoVar }
  TPathInfoVar = class(TScriptVariant)
  protected
    function GetValue: Variant; override;
    procedure SetValue(AValue: Variant); override;
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

  { TAuthorizationVar }
  TAuthorizationVar = class(TScriptVariant)
  protected
    function GetValue: Variant; override;
    procedure SetValue(AValue: Variant); override;
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

  { TStatusCodeVar }
  TStatusCodeVar = class(TScriptVariant)
  protected
    function GetValue: Variant; override;
    procedure SetValue(AValue: Variant); override;
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

  { TMethodVar }
  TMethodVar = class(TScriptVariant)
  protected
    function GetValue: Variant; override;
    procedure SetValue(AValue: Variant); override;
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

  { TProtocolVersionVar }
  TProtocolVersionVar = class(TScriptVariant)
  protected
    function GetValue: Variant; override;
    procedure SetValue(AValue: Variant); override;
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

  { TURLVar }
  TURLVar = class(TScriptVariant)
  protected
    function GetValue: Variant; override;
    procedure SetValue(AValue: Variant); override;
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

  { TResponseContentVar }
  TResponseContentVar = class(TScriptVariant)
  protected
    function GetValue: Variant; override;
    procedure SetValue(AValue: Variant); override;
  public
    constructor Create(AOwnerList: TScriptVariantList); override;
  end;

{ RegisterVariants }
procedure RegisterVariants(Vars: array of TScriptVariantClass);

var
  ScriptVariantList: TScriptVariantList;
  ScriptConstantList: TScriptVariantList;

implementation

uses CanApp;//, HTTPApp, CGIHTTP;

{ InitScriptVariantList }
procedure InitScriptVariantList;
begin
  ScriptVariantList := TScriptVariantList.Create;
  ScriptConstantList := TScriptVariantList.Create;
end;

{ DoneScriptVariantList }
procedure DoneScriptVariantList;
begin
  ScriptConstantList.Free;
  ScriptVariantList.Free;
end;

{ RegisterVariants }
procedure RegisterVariants(Vars: array of TScriptVariantClass);
var
  I: Integer;
begin
  for I := Low(Vars) to High(Vars) do
    ScriptVariantList.AddVariant(Vars[I]);
end;

{ RegisterScriptVariants }
procedure RegisterScriptVariants;
begin
  RegisterVariants([TTrueVar, TFalseVar, TMaxIntVar, TTitleVar, TRemoteAddrVar,
    TPathInfoVar, TAuthorizationVar, TStatusCodeVar, TMethodVar,
    TProtocolVersionVar, TURLVar, TResponseContentVar]);
  {
  ScriptVariantList.Add(TScriptVariant.Create('_Author', 'Nasir Senturk', idString));
  ScriptVariantList.Add(TScriptVariant.Create('_Server', 'Haysem Http Server', idString));
  ScriptVariantList.Add(TScriptVariant.Create('True', True, idUnknown));
  ScriptVariantList.Add(TScriptVariant.Create('False', False, idUnknown));
  ScriptVariantList.Add(TScriptVariant.Create('MaxInt', MaxInt, idInteger));
  ScriptVariantList.Add(TScriptVariant.Create('_pi', PI, idFloat));
  ScriptVariantList.Add(TScriptVariant.Create('_e', Exp(1.0), idFloat));
  //ScriptVariantList.Add(TScriptVariant.Create('Expires', -1.0, idFloat));
  }
end;

{ TScriptVariant }

constructor TScriptVariant.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create;
  FOwnerList := AOwnerList;
end;

destructor TScriptVariant.Destroy;
begin
  //WriteLn(Name, ' ', Variants.VarType(FValue));
  //if FValue is IUnknown then;
  FValue := Unassigned;
  inherited Destroy;
end;

function TScriptVariant.GetValue: Variant;
begin
  Result := FValue;
end;

procedure TScriptVariant.SetName(const AValue: string);
begin
  if not FOwnerList.IsDefined(AValue) then
    FName := AValue
  else raise Exception.CreateFmt('Variant %s redefined.', [AValue]);
end;

procedure TScriptVariant.SetValue(AValue: Variant);
begin
  FValue := AValue;
end;

{ TScriptVariantList }

function TScriptVariantList.Add(AVariant: TScriptVariant): Integer;
begin
  Result := inherited Add(AVariant);
end;

function TScriptVariantList.AddVar: TScriptVariant;
begin
  Result := TScriptVariant.Create(Self);
  Add( Result );
end;

function TScriptVariantList.AddVariant(
  VarClass: TScriptVariantClass): Integer;
begin
  Result := inherited Add(VarClass.Create(Self));
end;

function TScriptVariantList.CreateVar(const AName: string; AValue: Variant;
  AVarType: Integer): Integer;
var
  ScriptVar: TScriptVariant;
begin
  ScriptVar := TScriptVariant.Create(Self);
  with ScriptVar do
  begin
    Name := AName;
    Value := AValue;
    VarType := AVarType;
  end;
  Result := Add(ScriptVar);
end;

function TScriptVariantList.GetItem(Index: Integer): TScriptVariant;
begin
  Result := TScriptVariant(inherited Items[Index]);
end;

function TScriptVariantList.GetVariant(const Value: string): TScriptVariant;
var
  Item: TScriptVariant;
  Index: Integer;
begin
  Result := nil;
  for Index := 0 to Self.Count - 1 do
  begin
    Item := Self.Items[Index];
    if SameText( Value, Item.Name ) then
    begin
      Result := Item;
      Break;
    end;
  end;
end;

function TScriptVariantList.IndexOfName(const AName: string): Integer;
var
  I: Integer;
  Item: TScriptVariant;
begin
  Result := -1;
  for I := 0 to Self.Count - 1 do
  begin
    Item := Self.Items[I];
    if SameText(Item.Name, AName) then
    begin
      Result := I;
      Break;
    end;
  end;
end;

function TScriptVariantList.IsDefined(const Value: string): Boolean;
begin
  Result := Self.IndexOfName(Value) > -1;
end;

procedure TScriptVariantList.SetItem(Index: Integer;
  AVariant: TScriptVariant);
begin
  inherited Items[Index] := AVariant;
end;

{ TTrueVar }

constructor TTrueVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'True';
  Value := True;
  VarType := idBoolean;
end;

{ TFalseVar }

constructor TFalseVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'False';
  Value := False;
  VarType := idBoolean;
end;

{ TMaxIntVar }

constructor TMaxIntVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'MaxInt';
  Value := MaxInt;
  VarType := idInteger;
end;

{ TTitleVar }

constructor TTitleVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'Request_Title';
  VarType := idString;
end;

function TTitleVar.GetValue: Variant;
begin
  Result := Application.Response.Title;
end;

procedure TTitleVar.SetValue(AValue: Variant);
begin
  Application.Response.Title := AValue;
end;

{ TRemoteAddrVar }

constructor TRemoteAddrVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'Request_RemoteAddr';
  VarType := idString;
end;

function TRemoteAddrVar.GetValue: Variant;
begin
  Result := Application.Request.RemoteAddr;
end;

procedure TRemoteAddrVar.SetValue(AValue: Variant);
begin
  raise Exception.CreateFmt( '"%s" is read only property.', [ Name ] );
end;

{ TPathInfoVar }

constructor TPathInfoVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'Request_PathInfo';
  VarType := idString;
end;

function TPathInfoVar.GetValue: Variant;
begin
  Result := Application.Request.PathInfo;
end;

procedure TPathInfoVar.SetValue(AValue: Variant);
begin
  raise Exception.CreateFmt( '"%s" is read only property.', [ Name ] );
end;

{ TAuthorizationVar }

constructor TAuthorizationVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'Request_Authorization';
  VarType := idString;
end;

function TAuthorizationVar.GetValue: Variant;
begin
  Result := Application.Request.Authorization;
end;

procedure TAuthorizationVar.SetValue(AValue: Variant);
begin
  raise Exception.CreateFmt( '"%s" is read only property.', [ Name ] );
end;

{ TStatusCodeVar }

constructor TStatusCodeVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'Response_StatusCode';
  VarType := idInteger;
end;

function TStatusCodeVar.GetValue: Variant;
begin
  Result := Application.Response.StatusCode;
end;

procedure TStatusCodeVar.SetValue(AValue: Variant);
begin
  Application.Response.StatusCode := AValue;
end;

{ TMethodVar }

constructor TMethodVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'Request_Method';
  VarType := idString;
end;

function TMethodVar.GetValue: Variant;
begin
  Result := Application.Request.Method;
end;

procedure TMethodVar.SetValue(AValue: Variant);
begin
  inherited;
end;

{ TProtocolVersionVar }

constructor TProtocolVersionVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'Request_ProtocolVersion';
  VarType := idString;
end;

function TProtocolVersionVar.GetValue: Variant;
begin
  Result := Application.Request.ProtocolVersion;
end;

procedure TProtocolVersionVar.SetValue(AValue: Variant);
begin
  inherited;
end;

{ TURLVar }

constructor TURLVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'Request_URL';
  VarType := idString;
end;

function TURLVar.GetValue: Variant;
begin
  Result := Application.Request.URL;
end;

procedure TURLVar.SetValue(AValue: Variant);
begin
  inherited;
end;

{ TResponseContentVar }

constructor TResponseContentVar.Create(AOwnerList: TScriptVariantList);
begin
  inherited Create(AOwnerList);
  Name := 'Response_Content';
  VarType := idString;
end;

function TResponseContentVar.GetValue: Variant;
begin
  Result := Application.Response.Content;
end;

procedure TResponseContentVar.SetValue(AValue: Variant);
begin
  Application.Response.Content := AValue;
end;

initialization
  InitScriptVariantList;
  RegisterScriptVariants;

finalization
  DoneScriptVariantList;

end.
