unit StringUtils;

interface

uses SysUtils, Classes, Variants, ScriptFuncs, ScriptParser, SysObjList;

type
  { TStringListCreateFunc }
  TStringListCreateFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStringListAddFunc }
  TStringListAddFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStringListCountFunc }
  TStringListCountFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStringListGetFunc }
  TStringListGetFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStringListPutFunc }
  TStringListPutFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStringListSaveFunc }
  TStringListSaveFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStringListLoadFunc }
  TStringListLoadFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStringListClearFunc }
  TStringListClearFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStringListGetNamesFunc }
  TStringListGetNamesFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStringListGetValuesFunc }
  TStringListGetValuesFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStringListPutValuesFunc }
  TStringListPutValuesFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

  { TStringListSortFunc }
  TStringListSortFunc = class(TScriptFunction)
  public
    constructor Create(AOwnerList: TScriptFunctionList); override;
    function Execute: Variant; override;
  end;

implementation

{ RegisterStringFunctions }
procedure RegisterStringFunctions;
begin
  RegisterFunctions( [TStringListCreateFunc, TStringListAddFunc,
    TStringListCountFunc, TStringListGetFunc, TStringListPutFunc,
    TStringListSaveFunc, TStringListLoadFunc, TStringListClearFunc,
    TStringListGetNamesFunc, TStringListGetValuesFunc,
    TStringListPutValuesFunc, TStringListSortFunc] );
end;

{ TStringListCreateFunc }

constructor TStringListCreateFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_Create';
  ResultType := idObject;
end;

function TStringListCreateFunc.Execute: Variant;
var
  StringList: TStringList;
begin
  Result := Unassigned;
  StringList := TStringList.Create;
  ObjectResult := StringList;
  SystemObjectList.Add( StringList );
end;

{ TStringListAddFunc }

constructor TStringListAddFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_Add';
  ResultType := idInteger;
  with Params.AddParam do begin
    Name := '@StringList';
    ParamType := idObject;
  end;
  with Params.AddParam do begin
    Name := '@String';
    ParamType := idString;
  end;
end;

function TStringListAddFunc.Execute: Variant;
var
  StringList: TStringList;
  Value: string;
begin
  StringList := TStringList(Params[0].ObjectValue);
  Value := Params[1].Value;
  Result := StringList.Add( Value );
  Params[0].Value := Unassigned;
  Params[0].ObjectValue := nil;
  Params[1].Value := Unassigned;
end;

{ TStringListCountFunc }

constructor TStringListCountFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_Count';
  ResultType := idInteger;
  with Params.AddParam do begin
    Name := '@StringList';
    ParamType := idObject;
  end;
end;

function TStringListCountFunc.Execute: Variant;
var
  StringList: TStringList;
begin
  StringList := TStringList(Params[0].ObjectValue);
  Result := StringList.Count;
  Params[0].Value := Unassigned;
  Params[0].ObjectValue := nil;
end;

{ TStringListGetFunc }

constructor TStringListGetFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_Get';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@StringList';
    ParamType := idObject;
  end;
  with Params.AddParam do begin
    Name := '@Index';
    ParamType := idInteger;
  end;
end;

function TStringListGetFunc.Execute: Variant;
var
  StringList: TStringList;
  Index: Integer;
begin
  StringList := TStringList(Params[0].ObjectValue);
  Index := Params[1].Value;
  Result := StringList[Index];
  Params[0].Value := Unassigned;
  Params[0].ObjectValue := nil;
  Params[1].Value := Unassigned;
end;

{ TStringListPutFunc }

constructor TStringListPutFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_Put';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@StringList';
    ParamType := idObject;
  end;
  with Params.AddParam do begin
    Name := '@Index';
    ParamType := idInteger;
  end;
  with Params.AddParam do begin
    Name := '@String';
    ParamType := idString;
  end;
end;

function TStringListPutFunc.Execute: Variant;
var
  StringList: TStringList;
  Index: Integer;
  Value: string;
begin
  StringList := TStringList(Params[0].ObjectValue);
  Index := Params[1].Value;
  Value := Params[2].Value;
  StringList[Index] := Value;
  Result := Unassigned;
  Params[0].Value := Unassigned;
  Params[0].ObjectValue := nil;
  Params[1].Value := Unassigned;
  Params[2].Value := Unassigned;
end;

{ TStringListSaveFunc }

constructor TStringListSaveFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_Save';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@StringList';
    ParamType := idObject;
  end;
  with Params.AddParam do begin
    Name := '@FileName';
    ParamType := idString;
  end;
end;

function TStringListSaveFunc.Execute: Variant;
var
  StringList: TStringList;
  FileName: string;
begin
  StringList := TStringList(Params[0].ObjectValue);
  FileName := Params[1].Value;
  StringList.SaveToFile( FileName );
  Result := Unassigned;
  Params[0].Value := Unassigned;
  Params[0].ObjectValue := nil;
  Params[1].Value := Unassigned;
end;

{ TStringListLoadFunc }

constructor TStringListLoadFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_Load';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@StringList';
    ParamType := idObject;
  end;
  with Params.AddParam do begin
    Name := '@FileName';
    ParamType := idString;
  end;
end;

function TStringListLoadFunc.Execute: Variant;
var
  StringList: TStringList;
  FileName: string;
begin
  StringList := TStringList(Params[0].ObjectValue);
  FileName := Params[1].Value;
  StringList.LoadFromFile( FileName );
  Result := Unassigned;
  Params[0].Value := Unassigned;
  Params[0].ObjectValue := nil;
  Params[1].Value := Unassigned;
end;

{ TStringListClearFunc }

constructor TStringListClearFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_Clear';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@StringList';
    ParamType := idObject;
  end;
end;

function TStringListClearFunc.Execute: Variant;
var
  StringList: TStringList;
begin
  StringList := TStringList(Params[0].ObjectValue);
  Result := Unassigned;
  StringList.Clear;
  Params[0].Value := Unassigned;
  Params[0].ObjectValue := nil;
end;

{ TStringListGetNamesFunc }

constructor TStringListGetNamesFunc.Create(
  AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_GetNames';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@StringList';
    ParamType := idObject;
  end;
  with Params.AddParam do begin
    Name := '@Index';
    ParamType := idInteger;
  end;
end;

function TStringListGetNamesFunc.Execute: Variant;
var
  StringList: TStringList;
  Index: Integer;
begin
  StringList := TStringList(Params[0].ObjectValue);
  Index := Params[1].Value;
  Result := StringList.Names[Index];
  Params[0].Value := Unassigned;
  Params[0].ObjectValue := nil;
  Params[1].Value := Unassigned;
end;

{ TStringListGetValuesFunc }

constructor TStringListGetValuesFunc.Create(
  AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_GetValues';
  ResultType := idString;
  with Params.AddParam do begin
    Name := '@StringList';
    ParamType := idObject;
  end;
  with Params.AddParam do begin
    Name := '@Index';
    ParamType := idString;
  end;
end;

function TStringListGetValuesFunc.Execute: Variant;
var
  StringList: TStringList;
  Index: string;
begin
  StringList := TStringList(Params[0].ObjectValue);
  Index := Params[1].Value;
  Result := StringList.Values[Index];
  Params[0].Value := Unassigned;
  Params[0].ObjectValue := nil;
  Params[1].Value := Unassigned;
end;

{ TStringListPutValuesFunc }

constructor TStringListPutValuesFunc.Create(
  AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_PutValues';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@StringList';
    ParamType := idObject;
  end;
  with Params.AddParam do begin
    Name := '@Index';
    ParamType := idString;
  end;
  with Params.AddParam do begin
    Name := '@String';
    ParamType := idString;
  end;
end;

function TStringListPutValuesFunc.Execute: Variant;
var
  StringList: TStringList;
  Index: string;
  Value: string;
begin
  StringList := TStringList(Params[0].ObjectValue);
  Index := Params[1].Value;
  Value := Params[2].Value;
  StringList.Values[Index] := Value;
  Result := Unassigned;
  Params[0].Value := Unassigned;
  Params[0].ObjectValue := nil;
  Params[1].Value := Unassigned;
  Params[2].Value := Unassigned;
end;

{ TStringListSortFunc }

constructor TStringListSortFunc.Create(AOwnerList: TScriptFunctionList);
begin
  inherited Create(AOwnerList);
  Name := 'StringList_Sort';
  ResultType := idUnknown;
  with Params.AddParam do begin
    Name := '@StringList';
    ParamType := idObject;
  end;
end;

function TStringListSortFunc.Execute: Variant;
var
  StringList: TStringList;
begin
  StringList := Params[0].ObjectValue as TStringList;
  Result := Unassigned;
  StringList.Sort;
  Params[0].Value := Unassigned;
  Params[0].ObjectValue := nil;
end;

initialization
  RegisterStringFunctions;

finalization

end.
