unit SysObjList;

interface

uses SysUtils, Classes, Contnrs, Math, Variants, Windows;

type
  { TSystemObjectItem }
  TSystemObjectItem = class(TObject)
  private
    FValue: TObject;
  public
    constructor Create;
    destructor Destroy; override;
    property Value: TObject read FValue write FValue;
  end;

  { TSystemObjectList }
  TSystemObjectList = class(TObjectList)
  public
    function AddObject(AObject: TSystemObjectItem): Integer;
  end;

  { THandleList }
  THandleList = class(TList)
  protected
    procedure Notify(Ptr: Pointer; Action: TListNotification); override;
    function Get(Index: Integer): THandle;
    procedure Put(Index: Integer; Item: THandle);
  public
    function Add(Item: THandle): Integer;
    function Remove(Item: THandle): Integer;
    function IndexOf(Item: THandle): Integer;
    function IsValid(Item: THandle): Boolean;
    property Items[Index: Integer]: THandle read Get write Put; default;
  end;

var
  //SystemObjectList: TSystemObjectList;
  SystemObjectList: TObjectList;
  SystemHandleList: THandleList;

implementation

{ InitExecutionList }
procedure InitSystemObjectList;
begin
  //SystemObjectList := TSystemObjectList.Create;
  SystemObjectList := TObjectList.Create;
  SystemHandleList := THandleList.Create;
end;

{ DoneExecutionList }
procedure DoneSystemObjectList;
begin
  //WriteLn( 'Object List Count: ', SystemObjectList.Count );
  SystemHandleList.Free;
  SystemObjectList.Free;
end;

{ TSystemObjectItem }

constructor TSystemObjectItem.Create;
begin
  inherited Create;
end;

destructor TSystemObjectItem.Destroy;
begin
  if Assigned(FValue) then FreeAndNil(FValue);
  inherited Destroy;
end;

{ TSystemObjectList }

function TSystemObjectList.AddObject(AObject: TSystemObjectItem): Integer;
begin
  Result := inherited Add(AObject);
end;

{ THandleList }

function THandleList.Add(Item: THandle): Integer;
begin
  Result := inherited Add(Pointer(Item));
end;

function THandleList.Get(Index: Integer): THandle;
begin
  Result := THandle(inherited Get(Index));
end;

function THandleList.IndexOf(Item: THandle): Integer;
begin
  Result := inherited IndexOf(Pointer(Item));
end;

function THandleList.IsValid(Item: THandle): Boolean;
begin
  Result := IndexOf(Item) > -1;
end;

procedure THandleList.Notify(Ptr: Pointer; Action: TListNotification);
begin
  if Action = lnDeleted then
    if THandle(Ptr) <> INVALID_HANDLE_VALUE then
      CloseHandle(THandle(Ptr));
  inherited Notify(Ptr, Action);
end;

procedure THandleList.Put(Index: Integer; Item: THandle);
begin
  inherited Put(Index, Pointer(Item));
end;

function THandleList.Remove(Item: THandle): Integer;
begin
  Result := inherited Remove(Pointer(Item));
end;

initialization
  InitSystemObjectList;

finalization
  DoneSystemObjectList;

end.
